<?php
/**
 * The class to actually restrict the shipping methods
 *
 * @link       https://wpruby.com
 * @since      1.0.0
 *
 * @package    Restricted_Shipping_And_Payment_For_Woocommerce
 * @subpackage Restricted_Shipping_And_Payment_For_Woocommerce/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Restricted_Shipping_And_Payment_For_Woocommerce
 * @subpackage Restricted_Shipping_And_Payment_For_Woocommerce/public
 * @author     WPRuby <info@wpruby.com>
 */
class RSPW_Shipping_Filter extends RSPW_Filter {


	/**
	 * RSPW_Shipping_Filter constructor.
	 */
	public function __construct() {
		add_filter( 'woocommerce_update_cart_action_cart_updated', 'clear_notices_on_cart_update', 10, 1 );
		add_action( 'woocommerce_package_rates', array( $this, 'exclude_package_shipping_methods' ), 10, 2 );
	}

	/**
	 * @param $rates
	 * @param $package
	 *
	 * @return mixed
	 */
	public function exclude_package_shipping_methods( $rates, $package ) {
		$conditions = $this->get_conditions();

		$package = WC()->cart->get_cart_contents();

		foreach ( $rates as $key => $rate ) {
			foreach ( $conditions as $condition ) {
				if ( ! $this->is_rate_from_condition_method( $rate, $condition ) ) {
					continue;
				}
				if ( $this->is_condition_match( $condition, $package ) ) {
					$this->display_cart_messages( $condition );
					unset( $rates[ $key ] );
				}
			}
		}
		return $rates;
	}

	/**
	 * @param $rate
	 * @param $condition
	 *
	 * @return bool
	 */
	private function is_rate_from_condition_method( $rate, $condition ) {
		/** @var  WC_Shipping_Rate $rate */
		$method_id                  = $rate->get_method_id();
		$instance_id                = $rate->get_instance_id();
		$rate_id                    = sprintf( '%s:%s', $method_id, $instance_id );
		$condition_shipping_methods = get_post_meta( $condition->ID, 'excluded_shipping_methods', true );
		if ( is_array( $condition_shipping_methods ) ) {
			if ( in_array( $rate_id, $condition_shipping_methods, true ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @inheritDoc
	 */
	protected function get_filter_rules_key() {
		return 'shipping_condition_rules';
	}

	/**
	 * @inheritDoc
	 */
	protected function get_conditions_post_type() {
		return RSPW_SHIPPING_CONDITION;
	}

	/**
	 * Output a message
	 *
	 * @param string $title
	 * @param mixed $data
	 * @param string $type
	 */
	protected function debug( $title, $data = null, $type = 'notice' ) {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		if ( ! $data ) {
			$message = $title;
		} else {
			$message = sprintf( '%s: <pre>%s</pre>', $title, wp_json_encode( $data, JSON_PRETTY_PRINT ) );
		}

		wc_add_notice( $message, $type );
	}

	public function clear_notices_on_cart_update() {
		wc_clear_notices();
	}

	private function display_matched_condition( $condition ) {
		$data = array(
			'condition name' => $condition->post_title,
			'id'             => $condition->ID,
			'rules'          => $this->get_rules( $condition ),
		);
		$this->debug( 'Matched Condition', $data );
	}

	private function display_cart_messages( $condition ) {
		$cart_message      = get_post_meta( $condition->ID, 'cart_message', true );
		$cart_message_type = get_post_meta( $condition->ID, 'cart_message_type', true );

		if ( trim( $cart_message ) === '' ) {
			return;
		}

		if ( ! wc_has_notice( $cart_message, $cart_message_type ) ) {
			wc_add_notice( $cart_message, $cart_message_type );
		}
	}
}

new RSPW_Shipping_Filter();
